import os
import sys
import subprocess
import shlex
import shutil
import platform

__version__ = "0.6.1"

#https://stackabuse.com/how-to-print-colored-text-in-python/
#https://stackoverflow.com/questions/287871/how-do-i-print-colored-text-to-the-terminal
HEADER = '\033[95m'
OKBLUE = '\033[94m'
OKCYAN = '\033[96m'
OKGREEN = '\033[92m'
WARNING = '\033[1;30;43m'
FAIL = '\033[91m'
ENDC = '\033[0m'
BOLD = '\033[1m'
UNDERLINE = '\033[4m'

def print_err(msg):
    print(f'{FAIL}:{msg}{ENDC}')

def print_warning(msg):
    print(f'{WARNING}:{msg}{ENDC}')

def find_wasm_file():
    try:
        with open('go.mod') as f:
            return f.readlines()[0].strip().split(' ')[1] + '.wasm'
    except FileNotFoundError:
        return None

def run_tinygo():
    dir_name = os.path.dirname(os.path.realpath(__file__))
    dir_name = os.path.join(dir_name, "tinygo")
    tinygo = os.path.join(dir_name, 'bin/tinygo')
    if len(sys.argv) <= 1:
        return subprocess.call(tinygo, stdout=sys.stdout, stderr=sys.stderr)
    elif sys.argv[1] == "build":
        cmd = [tinygo]
        args = 'build -gc=leaking -target eosio -wasm-abi=generic -scheduler=none -opt z -tags=math_big_pure_go'
        args = shlex.split(args)
        cmd.extend(args)
        cmd.extend(sys.argv[2:])
        ret_code = subprocess.call(cmd, stdout=sys.stdout, stderr=sys.stderr)
        if not ret_code == 0:
            sys.exit(ret_code)

        wasm = find_wasm_file()
        if shutil.which('wasm-opt'):
            cmd = f'wasm-opt {wasm} -Oz --strip-debug -o {wasm}2'
            cmd = shlex.split(cmd)
            ret_code = subprocess.call(cmd, stdout=sys.stdout, stderr=sys.stderr)
            if not ret_code == 0:
                sys.exit(ret_code)
            shutil.move(f"{wasm}2",  f"{wasm}")
        else:
            print_warning('''
wasm-opt not found! Make sure the binary is in your PATH environment.
We use this tool to optimize the size of your contract's Wasm binary.
wasm-opt is part of the binaryen package. You can find detailed
installation instructions on https://github.com/WebAssembly/binaryen#tools.
There are ready-to-install packages for many platforms:
* Debian/Ubuntu: apt-get install binaryen
* Homebrew: brew install binaryen
* Arch Linux: pacman -S binaryen
* Windows: binary releases at https://github.com/WebAssembly/binaryen/releases''')
    else:
        cmd = sys.argv[:]
        cmd[0] = tinygo
        return subprocess.call(cmd, stdout=sys.stdout, stderr=sys.stderr)

def run_command(tool):
    dir_name = os.path.dirname(os.path.realpath(__file__))
    dir_name = os.path.join(dir_name, "tinygo")
    tinygo = os.path.join(dir_name, 'bin/tinygo')
    cmd = sys.argv[:]
    cmd[0] = tinygo
    if platform.system() == 'Windows':
        tool += '.exe'
    cmd.insert(1, tool)
    return subprocess.call(cmd, stdout=sys.stdout, stderr=sys.stderr)

def run_clang():
    return run_command('clang')

def run_wasm_ld():
    return run_command('wasm-ld')

def run_ld_lld():
    return run_command('ld.lld')

def run_dlltool():
    return run_command('dlltool')

def run_ranlib():
    return run_command('ranlib')

def run_lib():
    return run_command('lib')

def run_ar():
    return run_command('ar')

def run_eosio_cpp():
    dir_name = os.path.dirname(os.path.realpath(__file__))
    dir_name = os.path.join(dir_name, "eosio.cdt")
    eosio_cpp = os.path.join(dir_name, 'bin/eosio-cpp')
    cmd = sys.argv[:]
    cmd[0] = eosio_cpp
    return subprocess.call(cmd, stdout=sys.stdout, stderr=sys.stderr)



#!/usr/bin/env python3

# This file is placed in the public domain.

# This file converts a simple header file like this:
#
#    #include <stdio.h>
#    #include <stdlib.h>
#
# to an assembly file that defines stub functions:
#
#    .global _malloc
#    _malloc:
#
#    .global _printf
#    _printf:
#
# ...etc.

import argparse
import sys
from collections import OrderedDict

# If this fails, install the Clang python bindings.
# For example, on Debian that's something like:
#    apt-get install python3-clang-13
import clang.cindex

# For some information on the (sparsely documented) clang package for Python:
# https://eli.thegreenplace.net/2011/07/03/parsing-c-in-python-with-clang

def generateStubs(sysroot, outfile, target):
    # These symbols are not defined in the header files, but must be declared
    # anyway.
    symbols = {'dyld_stub_binder', '___bzero'}

    # Parse the src/libSystem.h file to get a list of functions declared in libSystem.
    index = clang.cindex.Index.create()
    args = args=['--sysroot='+sysroot, '-Werror', '--target='+target]
    tu = index.parse('src/libSystem.h', args=args)
    if len(tu.diagnostics):
        print('failed to extract function stubs using %s:' % args)
        for diag in tu.diagnostics:
            print(diag)
        sys.exit(1)
    for child in tu.cursor.get_children():
        if child.kind != clang.cindex.CursorKind.FUNCTION_DECL and child.kind != clang.cindex.CursorKind.VAR_DECL:
            continue
        if child.linkage != clang.cindex.LinkageKind.EXTERNAL:
            continue
        symbols.add(child.mangled_name)

    # Now write a stub assembly file that defines these files as stubs. They
    # can't be called, they're only provided to let the linker know that these
    # symbols live in libSystem.B.dylib.
    f = open(outfile, 'w')
    f.write('// GENERATED FILE - DO NOT EDIT\n')
    f.write('// This file was generated using generate-stubs.py with --target=%s\n' % target)
    for symbol in sorted(list(symbols)):
        f.write('\n.global %s\n%s:\n' % (symbol, symbol))
    f.close()


def main():
    parser = argparse.ArgumentParser(description='Create libSystem.s stubs')
    parser.add_argument('--target', type=str, default='x86_64-apple-macosx10.12.0', help='Clang --target flag')
    parser.add_argument('sysroot', type=str, help='path to sysroot directory')
    parser.add_argument('output', type=str, help='where to write assembly stubs')
    args = parser.parse_args()
    generateStubs(args.sysroot, args.output, args.target)

if __name__ == '__main__':
    main()
